# Generated by Django 5.2.4 on 2025-12-19 10:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history4feed', '0008_job_extra_data_alter_fulltextjob_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='feed',
            name='use_scrapfly_asp',
            field=models.BooleanField(default=False),
        ),
    ]
